@@ -411,6 +411,9 @@ calculate_changed_events (GcalTimeline            *self,
   for (i = 0; events_to_remove && i < events_to_remove->len; i++)
     {
       GcalEvent *event = g_ptr_array_index (events_to_remove, i);
+      /* Do not re-remove multiday events that are part of new range */
+      if (gcal_event_is_multiday (event) && gcal_event_overlaps (event, new_range))
+        continue;
 
       GCAL_TRACE_MSG ("Removing event from subscriber %s due to time range change (event: '%s' (%s))",
                       G_OBJECT_TYPE_NAME (subscriber),
@@ -423,6 +426,9 @@ calculate_changed_events (GcalTimeline            *self,
   for (i = 0; events_to_add && i < events_to_add->len; i++)
     {
       GcalEvent *event = g_ptr_array_index (events_to_add, i);
+      /* Do not re-add multiday events that were part of old range */
+      if (gcal_event_is_multiday (event) && gcal_event_overlaps (event, old_range))
+        continue;
 
       GCAL_TRACE_MSG ("Queueing event addition for subscriber %s (event: '%s' (%s))",
                       G_OBJECT_TYPE_NAME (subscriber),
@@ -556,17 +562,29 @@ calendar_monitor_update_events_cb (GcalCalendarMonitor *monitor,
 
   for (guint i = 0; i < events->len; i++)
     {
+      g_autoptr (GPtrArray) old_subscribers_at_range = NULL;
       g_autoptr (GPtrArray) subscribers_at_range = NULL;
       GcalRange *event_range;
+      GcalRange *old_event_range;
       GcalEvent *old_event;
       GcalEvent *event;
 
       event = g_ptr_array_index (events, i);
       old_event = g_ptr_array_index (old_events, i);
       event_range = gcal_event_get_range (event);
+      old_event_range = gcal_event_get_range (old_event);
 
       /* Add to all subscribers within the event range */
       subscribers_at_range = gcal_range_tree_get_data_at_range (self->subscriber_ranges, event_range);
+      old_subscribers_at_range = gcal_range_tree_get_data_at_range (self->subscriber_ranges, old_event_range);
+
+      for (guint j = 0; old_subscribers_at_range && j < old_subscribers_at_range->len; j++)
+        {
+          GcalTimelineSubscriber *old_subscriber = g_ptr_array_index (old_subscribers_at_range, j);
+
+          if (!g_ptr_array_find (subscribers_at_range, old_subscriber, NULL))
+            queue_event_data (self, REMOVE_EVENT, old_subscriber, old_event, NULL, FALSE);
+        }
 
       queue_event_data (self, UPDATE_EVENT, NULL, event, old_event, TRUE);
 
@@ -574,7 +592,10 @@ calendar_monitor_update_events_cb (GcalCalendarMonitor *monitor,
         {
           GcalTimelineSubscriber *subscriber = g_ptr_array_index (subscribers_at_range, j);
 
-          queue_event_data (self, UPDATE_EVENT, subscriber, event, old_event, FALSE);
+          if (g_ptr_array_find (old_subscribers_at_range, subscriber, NULL))
+            queue_event_data (self, UPDATE_EVENT, subscriber, event, old_event, FALSE);
+          else
+            queue_event_data (self, ADD_EVENT, subscriber, event, NULL, FALSE);
         }
 
     }
