@@ -27,7 +27,6 @@
 #include "gcal-context.h"
 #include "gcal-calendar-management-page.h"
 #include "gcal-debug.h"
-#include "gcal-file-chooser-button.h"
 #include "gcal-new-calendar-page.h"
 #include "gcal-source-discoverer.h"
 #include "gcal-utils.h"
@@ -44,12 +43,11 @@ typedef enum
 
 struct _GcalNewCalendarPage
 {
-  AdwBin              parent;
+  AdwNavigationPage   parent;
 
   GtkWidget          *add_button;
   GtkEntry           *calendar_address_entry;
   EntryState          calendar_address_entry_state;
-  GcalFileChooserButton *calendar_file_chooser_button;
   AdwMessageDialog   *credentials_dialog;
   GtkEntry           *credentials_password_entry;
   GtkEntry           *credentials_user_entry;
@@ -75,9 +73,11 @@ static void          sources_discovered_cb                       (GObject
                                                                   GAsyncResult      *result,
                                                                   gpointer           user_data);
 
+static void          cancel_validation_discover                               (GcalNewCalendarPage *self);
+
 static void          gcal_calendar_management_page_iface_init    (GcalCalendarManagementPageInterface *iface);
 
-G_DEFINE_TYPE_WITH_CODE (GcalNewCalendarPage, gcal_new_calendar_page, ADW_TYPE_BIN,
+G_DEFINE_TYPE_WITH_CODE (GcalNewCalendarPage, gcal_new_calendar_page, ADW_TYPE_NAVIGATION_PAGE,
                          G_IMPLEMENT_INTERFACE (GCAL_TYPE_CALENDAR_MANAGEMENT_PAGE,
                                                 gcal_calendar_management_page_iface_init))
 
@@ -147,10 +147,10 @@ update_add_button (GcalNewCalendarPage *self)
   gtk_widget_set_sensitive (self->add_button, valid);
 
   n_calendars = 0;
-  if (self->local_source)
-    n_calendars++;
   if (self->remote_sources)
     n_calendars += self->remote_sources->len;
+  else if (self->local_source)
+    n_calendars++;
 
   if (n_calendars > 0)
     {
@@ -162,7 +162,7 @@ update_add_button (GcalNewCalendarPage *self)
     }
   else
     {
-      add_button_label = g_strdup (_("Add Calendar"));
+      add_button_label = g_strdup (C_("button", "Add Calendar"));
     }
 
   gtk_button_set_label (GTK_BUTTON (self->add_button), add_button_label);
@@ -209,24 +209,54 @@ static void
 toggle_url_entry_pulsing (GcalNewCalendarPage *self,
                           gboolean             pulsing)
 {
-  gtk_entry_set_progress_fraction (self->calendar_address_entry, 0);
-
   if (pulsing && self->calendar_address_id == 0)
-    self->calendar_address_id = g_timeout_add (ENTRY_PROGRESS_TIMEOUT, pulse_web_entry, self);
+    {
+      gtk_entry_set_progress_fraction (self->calendar_address_entry, 0.1);
+      self->calendar_address_id = g_timeout_add (ENTRY_PROGRESS_TIMEOUT, pulse_web_entry, self);
+    }
   else if (!pulsing && self->calendar_address_id != 0)
-    g_clear_handle_id (&self->calendar_address_id, g_source_remove);
+    {
+      gtk_entry_set_progress_fraction (self->calendar_address_entry, 0.0);
+      g_clear_handle_id (&self->calendar_address_id, g_source_remove);
+    }
 }
 
 static void
 update_url_entry_state (GcalNewCalendarPage *self,
-                        EntryState           state)
+                        EntryState           state,
+                        const gchar         *error_message)
 {
   self->calendar_address_entry_state = state;
 
   if (state == ENTRY_STATE_INVALID)
-    gtk_widget_add_css_class (GTK_WIDGET (self->calendar_address_entry), "error");
+    {
+      gtk_widget_add_css_class (GTK_WIDGET (self->calendar_address_entry), "error");
+      if (error_message)
+        {
+          g_autofree gchar *multiline_message = NULL;
+          g_auto(GStrv) split_message = NULL;
+
+          split_message = g_strsplit (error_message, ": ", -1);
+          multiline_message = g_strjoinv (":\n", split_message);
+
+          gtk_entry_set_icon_from_icon_name (self->calendar_address_entry,
+                                             GTK_ENTRY_ICON_SECONDARY,
+                                             "dialog-error");
+          gtk_entry_set_icon_tooltip_text (self->calendar_address_entry,
+                                           GTK_ENTRY_ICON_SECONDARY,
+                                           multiline_message);
+        }
+    }
   else
-    gtk_widget_remove_css_class (GTK_WIDGET (self->calendar_address_entry), "error");
+    {
+      gtk_widget_remove_css_class (GTK_WIDGET (self->calendar_address_entry), "error");
+      gtk_entry_set_icon_from_icon_name (self->calendar_address_entry,
+                                         GTK_ENTRY_ICON_SECONDARY,
+                                         NULL);
+      gtk_entry_set_icon_tooltip_text (self->calendar_address_entry,
+                                       GTK_ENTRY_ICON_SECONDARY,
+                                       NULL);
+    }
 
   update_add_button (self);
   toggle_url_entry_pulsing (self, state == ENTRY_STATE_VALIDATING);
@@ -253,7 +283,7 @@ discover_sources (GcalNewCalendarPage *self)
                                   self->cancellable,
                                   sources_discovered_cb,
                                   self);
-  update_url_entry_state (self, ENTRY_STATE_VALIDATING);
+  update_url_entry_state (self, ENTRY_STATE_VALIDATING, NULL);
 
   GCAL_EXIT;
 }
@@ -262,49 +292,6 @@ discover_sources (GcalNewCalendarPage *self)
  * Callbacks
  */
 
-static void
-on_file_chooser_button_file_changed_cb (GcalFileChooserButton *chooser,
-                                        GParamSpec            *pspec,
-                                        GcalNewCalendarPage   *self)
-{
-  g_autofree gchar *display_name = NULL;
-  g_autoptr (ESource) source = NULL;
-  g_autoptr (GFile) file = NULL;
-  ESourceExtension *ext;
-
-  GCAL_ENTRY;
-
-  file = gcal_file_chooser_button_get_file (chooser);
-
-  if (!file)
-    GCAL_RETURN ();
-
-  /* Create the new source and add the needed extensions */
-  source = e_source_new (NULL, NULL, NULL);
-  e_source_set_parent (source, "local-stub");
-
-  ext = e_source_get_extension (source, E_SOURCE_EXTENSION_CALENDAR);
-  e_source_backend_set_backend_name (E_SOURCE_BACKEND (ext), "local");
-
-  ext = e_source_get_extension (source, E_SOURCE_EXTENSION_LOCAL_BACKEND);
-  e_source_local_set_custom_file (E_SOURCE_LOCAL (ext), file);
-
-  /* update the source properties */
-  display_name = calendar_path_to_name_suggestion (file);
-  e_source_set_display_name (source, display_name);
-
-  /* TODO: report errors */
-  gcal_manager_save_source (gcal_context_get_manager (self->context), source);
-
-  gcal_calendar_management_page_switch_page (GCAL_CALENDAR_MANAGEMENT_PAGE (self),
-                                             "calendars",
-                                             NULL);
-
-  gcal_file_chooser_button_set_file (self->calendar_file_chooser_button, NULL);
-
-  GCAL_EXIT;
-}
-
 static gboolean
 pulse_web_entry (gpointer data)
 {
@@ -342,10 +329,16 @@ sources_discovered_cb (GObject      *source_object,
           gtk_window_set_transient_for (GTK_WINDOW (self->credentials_dialog), GTK_WINDOW (root));
           gtk_window_present (GTK_WINDOW (self->credentials_dialog));
         }
-      else
+      /*
+       * If the operation was cancelled by the user, no error message
+       * or error style class will be applied
+       */
+      else if (!g_error_matches (error,
+                                 G_IO_ERROR,
+                                 G_IO_ERROR_CANCELLED))
         {
           g_warning ("Error finding sources: %s", error->message);
-          update_url_entry_state (self, ENTRY_STATE_INVALID);
+          update_url_entry_state (self, ENTRY_STATE_INVALID, error->message);
         }
       GCAL_RETURN ();
     }
@@ -353,7 +346,7 @@ sources_discovered_cb (GObject      *source_object,
   g_debug ("Found %u sources", sources->len);
 
   self->remote_sources = g_steal_pointer (&sources);
-  update_url_entry_state (self, ENTRY_STATE_VALID);
+  update_url_entry_state (self, ENTRY_STATE_VALID, NULL);
 
   GCAL_EXIT;
 }
@@ -366,6 +359,7 @@ validate_url_cb (gpointer data)
 
   GCAL_ENTRY;
 
+  update_url_entry_state (self, ENTRY_STATE_VALIDATING, NULL);
   self->validate_url_resource_id = 0;
 
   guri = g_uri_parse (gtk_editable_get_text (GTK_EDITABLE (self->calendar_address_entry)), SOUP_HTTP_URI_FLAGS | G_URI_FLAGS_PARSE_RELAXED, NULL);
@@ -376,7 +370,7 @@ validate_url_cb (gpointer data)
     }
   else
     {
-      update_url_entry_state (self, ENTRY_STATE_INVALID);
+      update_url_entry_state (self, ENTRY_STATE_INVALID, _("The URL you have entered appears to be invalid."));
       g_debug ("Invalid URL passed");
     }
 
@@ -391,10 +385,6 @@ on_add_button_clicked_cb (GtkWidget           *button,
 
   manager = gcal_context_get_manager (self->context);
 
-  /* Commit the new source */
-  if (self->local_source)
-    gcal_manager_save_source (manager, self->local_source);
-
   /* Commit each new remote source */
   if (self->remote_sources)
     {
@@ -406,6 +396,11 @@ on_add_button_clicked_cb (GtkWidget           *button,
       g_clear_pointer (&self->remote_sources, g_ptr_array_unref);
     }
 
+  else
+    {
+      gcal_manager_save_source (manager, self->local_source);
+    }
+
   gcal_calendar_management_page_switch_page (GCAL_CALENDAR_MANAGEMENT_PAGE (self),
                                              "calendars",
                                              NULL);
@@ -415,6 +410,7 @@ static void
 on_calendar_address_activated_cb (GtkEntry            *entry,
                                   GcalNewCalendarPage *self)
 {
+  cancel_validation_discover (self);
   validate_url_cb (self);
 }
 
@@ -426,26 +422,19 @@ on_credentials_dialog_response_cb (AdwMessageDialog    *dialog,
   if (g_str_equal (response, "connect"))
     discover_sources (self);
   else if (g_str_equal (response, "cancel"))
-    update_url_entry_state (self, ENTRY_STATE_INVALID);
+    update_url_entry_state (self, ENTRY_STATE_INVALID,
+                            _("A valid user and password are required to connect to this calendar."));
 
   gtk_editable_set_text (GTK_EDITABLE (self->credentials_user_entry), "");
   gtk_editable_set_text (GTK_EDITABLE (self->credentials_password_entry), "");
 }
 
 static void
-on_url_entry_text_changed_cb (GtkEntry            *entry,
-                              GParamSpec          *pspec,
-                              GcalNewCalendarPage *self)
+cancel_validation_discover (GcalNewCalendarPage *self)
 {
-  const gchar* text;
-
-  GCAL_ENTRY;
-
-  text = gtk_editable_get_text (GTK_EDITABLE (entry));
-
   if (self->calendar_address_id != 0)
     {
-      gtk_entry_set_progress_fraction (entry, 0);
+      gtk_entry_set_progress_fraction (self->calendar_address_entry, 0);
       g_clear_handle_id (&self->calendar_address_id, g_source_remove);
     }
 
@@ -455,19 +444,40 @@ on_url_entry_text_changed_cb (GtkEntry            *entry,
   if (self->validate_url_resource_id != 0)
     g_clear_handle_id (&self->validate_url_resource_id, g_source_remove);
 
+  /* Cancel a validation in case it is on-going */
+  if (self->calendar_address_entry_state == ENTRY_STATE_VALIDATING)
+    {
+      g_cancellable_cancel (self->cancellable);
+      g_clear_object (&self->cancellable);
+      self->cancellable = g_cancellable_new ();
+    }
+}
+
+static void
+on_url_entry_text_changed_cb (GtkEntry            *entry,
+                              GParamSpec          *pspec,
+                              GcalNewCalendarPage *self)
+{
+  const gchar *text;
+
+  GCAL_ENTRY;
+
+  text = gtk_editable_get_text (GTK_EDITABLE (entry));
+
+  cancel_validation_discover (self);
+
   if (text && g_utf8_strlen (text, -1) > 0)
     {
       /*
-       * At first, don't bother the user with the login prompt. Only prompt it when
-       * it fails.
+       * Only validate the text after the timeout, as we don't want to bother them
+       * as soon as they type into the entry.
        */
-      self->validate_url_resource_id = g_timeout_add (500, validate_url_cb, self);
-      update_url_entry_state (self, ENTRY_STATE_VALIDATING);
+      self->validate_url_resource_id = g_timeout_add (750, validate_url_cb, self);
     }
   else
     {
       gtk_entry_set_progress_fraction (self->calendar_address_entry, 0);
-      update_url_entry_state (self, ENTRY_STATE_EMPTY);
+      update_url_entry_state (self, ENTRY_STATE_EMPTY, NULL);
     }
 
   GCAL_EXIT;
@@ -504,18 +514,6 @@ on_web_description_label_link_activated_cb (GtkLabel            *label,
  * GcalCalendarManagementPage iface
  */
 
-static const gchar*
-gcal_new_calendar_page_get_name (GcalCalendarManagementPage *page)
-{
-  return "new-calendar";
-}
-
-static const gchar*
-gcal_new_calendar_page_get_title (GcalCalendarManagementPage *page)
-{
-  return _("New Calendar");
-}
-
 static void
 gcal_new_calendar_page_deactivate (GcalCalendarManagementPage *page)
 {
@@ -540,8 +538,6 @@ gcal_new_calendar_page_deactivate (GcalCalendarManagementPage *page)
 static void
 gcal_calendar_management_page_iface_init (GcalCalendarManagementPageInterface *iface)
 {
-  iface->get_name = gcal_new_calendar_page_get_name;
-  iface->get_title = gcal_new_calendar_page_get_title;
   iface->deactivate = gcal_new_calendar_page_deactivate;
 }
 
@@ -626,13 +622,10 @@ gcal_new_calendar_page_class_init (GcalNewCalendarPageClass *klass)
 
   g_object_class_override_property (object_class, PROP_CONTEXT, "context");
 
-  g_type_ensure (GCAL_TYPE_FILE_CHOOSER_BUTTON);
-
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/calendar/ui/gui/calendar-management/gcal-new-calendar-page.ui");
 
   gtk_widget_class_bind_template_child (widget_class, GcalNewCalendarPage, add_button);
   gtk_widget_class_bind_template_child (widget_class, GcalNewCalendarPage, calendar_address_entry);
-  gtk_widget_class_bind_template_child (widget_class, GcalNewCalendarPage, calendar_file_chooser_button);
   gtk_widget_class_bind_template_child (widget_class, GcalNewCalendarPage, credentials_dialog);
   gtk_widget_class_bind_template_child (widget_class, GcalNewCalendarPage, credentials_password_entry);
   gtk_widget_class_bind_template_child (widget_class, GcalNewCalendarPage, credentials_user_entry);
@@ -644,7 +637,6 @@ gcal_new_calendar_page_class_init (GcalNewCalendarPageClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, on_add_button_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_calendar_address_activated_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_credentials_dialog_response_cb);
-  gtk_widget_class_bind_template_callback (widget_class, on_file_chooser_button_file_changed_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_local_calendar_name_row_text_changed_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_local_calendar_color_button_rgba_changed_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_url_entry_text_changed_cb);
