@@ -39,16 +39,17 @@ typedef struct
 
 struct _GcalImportDialog
 {
-  AdwWindow           parent;
+  AdwDialog           parent;
 
+  GtkBox             *calendars_box;
   GtkWidget          *cancel_button;
   AdwComboRow        *calendar_combo_row;
   AdwPreferencesGroup *files_group;
-  GtkHeaderBar       *headerbar;
+  AdwHeaderBar       *headerbar;
   GtkWidget          *import_button;
   GtkWidget          *placeholder_spinner;
-  AdwPreferencesPage *preferences_page;
   GtkSizeGroup       *title_sizegroup;
+  AdwToastOverlay    *toast_overlay;
 
   GList              *rows;
   GcalCalendar       *selected_calendar;
@@ -64,7 +65,7 @@ static void          on_import_row_file_loaded_cb                 (GcalImportFil
                                                                    GPtrArray         *events,
                                                                    GcalImportDialog  *self);
 
-G_DEFINE_TYPE (GcalImportDialog, gcal_import_dialog, ADW_TYPE_WINDOW)
+G_DEFINE_TYPE (GcalImportDialog, gcal_import_dialog, ADW_TYPE_DIALOG)
 
 enum
 {
@@ -94,39 +95,94 @@ import_data_free (gpointer data)
   g_free (import_data);
 }
 
+static gboolean
+find_calendar (GListModel   *model,
+               GcalCalendar *calendar,
+               guint        *out_position)
+{
+  g_assert (out_position != NULL);
+  g_assert (G_IS_LIST_MODEL (model));
+  g_assert (GCAL_IS_CALENDAR (calendar));
+
+  for (guint i = 0; i < g_list_model_get_n_items (model); i++)
+    {
+      GcalCalendar *aux = g_list_model_get_item (model, i);
+      g_assert (GCAL_IS_CALENDAR (aux));
+      if (aux == calendar)
+        {
+          *out_position = i;
+          return TRUE;
+        }
+    }
+  return FALSE;
+}
+
 static void
 update_default_calendar (GcalImportDialog *self)
 {
+  g_autoptr (GListModel) calendars = NULL;
   GcalCalendar *default_calendar;
   GcalManager *manager;
-  GListModel *calendars;
   guint position;
 
   manager = gcal_context_get_manager (self->context);
-  calendars = gcal_manager_get_calendars_model (manager);
+  calendars = gcal_create_writable_calendars_model (manager);
   default_calendar = gcal_manager_get_default_calendar (manager);
 
-  if (default_calendar && g_list_store_find (G_LIST_STORE (calendars), default_calendar, &position))
+  if (default_calendar && find_calendar (calendars, default_calendar, &position))
     adw_combo_row_set_selected (self->calendar_combo_row, position);
 }
 
 static void
 setup_calendars (GcalImportDialog *self)
 {
+  g_autoptr (GListModel) calendars = NULL;
   GcalManager *manager;
 
   g_assert (self->context != NULL);
 
   manager = gcal_context_get_manager (self->context);
+  calendars = gcal_create_writable_calendars_model (manager);
 
   // TODO: sort model
 
-  adw_combo_row_set_model (self->calendar_combo_row, gcal_manager_get_calendars_model (manager));
+  adw_combo_row_set_model (self->calendar_combo_row, calendars);
   update_default_calendar (self);
 
   g_signal_connect_object (manager, "notify::default-calendar", G_CALLBACK (update_default_calendar), self, G_CONNECT_SWAPPED);
 }
 
+static void
+add_file (GcalImportDialog *self,
+          GFile            *file,
+          gboolean          multiple_files)
+{
+  AdwPreferencesGroup *group;
+  GtkWidget *row;
+  g_autofree gchar *basename = NULL;
+
+  GCAL_ENTRY;
+
+  row = gcal_import_file_row_new (self->context, file, self->title_sizegroup);
+  g_signal_connect (row, "file-loaded", G_CALLBACK (on_import_row_file_loaded_cb), self);
+
+  group = ADW_PREFERENCES_GROUP (adw_preferences_group_new ());
+  adw_preferences_group_add (group, row);
+
+  if (multiple_files)
+    {
+      basename = g_file_get_basename (file);
+      adw_preferences_group_set_title (group, basename);
+    }
+
+  gtk_box_append (self->calendars_box, GTK_WIDGET (group));
+  self->rows = g_list_prepend (self->rows, row);
+
+  gtk_widget_set_visible (self->placeholder_spinner, FALSE);
+
+  GCAL_EXIT;
+}
+
 static void
 setup_files (GcalImportDialog  *self,
              GFile            **files,
@@ -137,30 +193,30 @@ setup_files (GcalImportDialog  *self,
   GCAL_ENTRY;
 
   self->n_files = n_files;
-
   for (i = 0; i < n_files; i++)
+    add_file (self, files[i], n_files > 1);
+
+  GCAL_EXIT;
+}
+
+static void
+setup_files_list (GcalImportDialog *self,
+                  GList            *list)
+{
+  gint n_files = 0;
+  gboolean has_multiple = list->next != NULL;
+
+  GCAL_ENTRY;
+
+  has_multiple = list->next != NULL;
+  for (GList *iter = list; iter != NULL; iter = iter->next)
     {
-      AdwPreferencesGroup *group;
-      GtkWidget *row;
-
-      row = gcal_import_file_row_new (files[i], self->title_sizegroup);
-      g_signal_connect (row, "file-loaded", G_CALLBACK (on_import_row_file_loaded_cb), self);
-
-      group = ADW_PREFERENCES_GROUP (adw_preferences_group_new ());
-      adw_preferences_group_add (group, row);
-
-      if (n_files > 1)
-        {
-          g_autofree gchar *basename = g_file_get_basename (files[i]);
-          adw_preferences_group_set_title (group, basename);
-        }
-
-      adw_preferences_page_add (self->preferences_page, group);
-      self->rows = g_list_prepend (self->rows, row);
-
-      gtk_widget_set_visible (self->placeholder_spinner, FALSE);
+      n_files++;
+      add_file (self, G_FILE (iter->data), has_multiple);
     }
 
+  self->n_files = n_files;
+
   GCAL_EXIT;
 }
 
@@ -185,7 +241,7 @@ on_events_created_cb (GObject      *source_object,
   if (error)
     g_warning ("Error creating events: %s", error->message);
 
-  gtk_window_destroy (GTK_WINDOW (self));
+  adw_dialog_close (ADW_DIALOG (self));
 
   GCAL_EXIT;
 }
@@ -202,29 +258,23 @@ on_calendar_combo_row_selected_item_changed_cb (AdwComboRow      *combo_row,
   GCAL_EXIT;
 }
 
-static void
-on_cancel_button_clicked_cb (GtkButton        *button,
-                             GcalImportDialog *self)
-{
-  gtk_window_destroy (GTK_WINDOW (self));
-}
-
 static void
 on_combo_row_signal_factory_setup_cb (GtkSignalListItemFactory *factory,
                                       GtkListItem              *item,
                                       GcalImportDialog         *self)
 {
   GtkWidget *label;
-  GtkWidget *icon;
+  GtkWidget *color_icon;
+  GtkWidget *visibility_status_icon;
   GtkWidget *grid;
 
   grid = gtk_grid_new ();
   gtk_grid_set_column_spacing (GTK_GRID (grid), 6);
 
-  icon = gtk_image_new ();
-  gtk_widget_add_css_class (icon, "calendar-color-image");
-  g_object_set_data (G_OBJECT (grid), "icon", icon);
-  gtk_grid_attach (GTK_GRID (grid), icon, 0, 0, 1, 2);
+  color_icon = gtk_image_new ();
+  gtk_widget_add_css_class (color_icon, "calendar-color-image");
+  g_object_set_data (G_OBJECT (grid), "color_icon", color_icon);
+  gtk_grid_attach (GTK_GRID (grid), color_icon, 0, 0, 1, 2);
 
   label = gtk_label_new (NULL);
   gtk_label_set_xalign (GTK_LABEL (label), 0.0);
@@ -238,6 +288,13 @@ on_combo_row_signal_factory_setup_cb (GtkSignalListItemFactory *factory,
   g_object_set_data (G_OBJECT (grid), "subtitle", label);
   gtk_grid_attach (GTK_GRID (grid), label, 1, 1, 1, 1);
 
+  visibility_status_icon = gtk_image_new ();
+
+  g_object_set_data (G_OBJECT (grid), "visibility_status_icon", visibility_status_icon);
+  gtk_widget_set_hexpand (visibility_status_icon, TRUE);
+  gtk_widget_set_halign (visibility_status_icon, GTK_ALIGN_END);
+  gtk_grid_attach (GTK_GRID (grid), visibility_status_icon, 2, 0, 1, 2);
+
   gtk_list_item_set_child (item, grid);
 }
 
@@ -250,25 +307,42 @@ on_combo_row_signal_factory_bind_cb (GtkSignalListItemFactory *factory,
   g_autofree gchar *parent_name = NULL;
   GcalCalendar *calendar;
   GtkWidget *label;
-  GtkWidget *icon;
+  GtkWidget *color_icon_widget;
   GtkWidget *grid;
 
   calendar = gtk_list_item_get_item (item);
 
   grid = gtk_list_item_get_child (item);
 
+  color_icon_widget = g_object_get_data (G_OBJECT (grid), "color_icon");
   color_paintable = get_circle_paintable_from_color (gcal_calendar_get_color (calendar), 16);
+  gtk_image_set_from_paintable (GTK_IMAGE (color_icon_widget), color_paintable);
+
+  /* Show indicator icons for calendars that are not shown in the views */
+  if (!gcal_calendar_get_visible (calendar))
+    {
+      g_autofree gchar *accessible_description = NULL;
+      g_autoptr (GIcon) visibility_icon = NULL;
+      GtkWidget *visibility_status_icon_widget;
+
+      visibility_icon = g_themed_icon_new ("eye-not-looking-symbolic");
+      visibility_status_icon_widget = g_object_get_data (G_OBJECT (grid), "visibility_status_icon");
+      gtk_image_set_from_gicon (GTK_IMAGE (visibility_status_icon_widget), visibility_icon);
+
+      accessible_description = g_strdup (_("This calendar is hidden"));
+      gtk_accessible_update_property (GTK_ACCESSIBLE (visibility_status_icon_widget),
+                                      GTK_ACCESSIBLE_PROPERTY_DESCRIPTION, accessible_description, -1);
+    }
+
+  label = g_object_get_data (G_OBJECT (grid), "title");
+  gtk_label_set_label (GTK_LABEL (label), gcal_calendar_get_name (calendar));
+
+  /* Get a colored version of the calendar source's name, to use as a subtitle */
   get_source_parent_name_color (gcal_context_get_manager (self->context),
                                 gcal_calendar_get_source (calendar),
                                 &parent_name,
                                 NULL);
 
-  icon = g_object_get_data (G_OBJECT (grid), "icon");
-  gtk_image_set_from_paintable (GTK_IMAGE (icon), color_paintable);
-
-  label = g_object_get_data (G_OBJECT (grid), "title");
-  gtk_label_set_label (GTK_LABEL (label), gcal_calendar_get_name (calendar));
-
   label = g_object_get_data (G_OBJECT (grid), "subtitle");
   gtk_label_set_label (GTK_LABEL (label), parent_name);
 
@@ -397,7 +471,7 @@ on_import_row_file_loaded_cb (GcalImportFileRow *row,
                                         "Import %d events",
                                         self->n_events),
                            self->n_events);
-  gtk_window_set_title (GTK_WINDOW (self), title);
+  adw_dialog_set_title (ADW_DIALOG (self), title);
 
   gtk_widget_set_visible (GTK_WIDGET (row), TRUE);
 
@@ -409,6 +483,16 @@ on_import_row_file_loaded_cb (GcalImportFileRow *row,
  * GObject overrides
  */
 
+static void
+gcal_import_dialog_constructed (GObject *object)
+{
+  GcalImportDialog *self = (GcalImportDialog *)object;
+
+  G_OBJECT_CLASS (gcal_import_dialog_parent_class)->constructed (object);
+
+  setup_calendars (self);
+}
+
 static void
 gcal_import_dialog_finalize (GObject *object)
 {
@@ -455,7 +539,6 @@ gcal_import_dialog_set_property (GObject      *object,
     case PROP_CONTEXT:
       g_assert (self->context == NULL);
       self->context = g_value_dup_object (value);
-      setup_calendars (self);
       break;
 
     default:
@@ -469,6 +552,7 @@ gcal_import_dialog_class_init (GcalImportDialogClass *klass)
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
+  object_class->constructed = gcal_import_dialog_constructed;
   object_class->finalize = gcal_import_dialog_finalize;
   object_class->get_property = gcal_import_dialog_get_property;
   object_class->set_property = gcal_import_dialog_set_property;
@@ -494,11 +578,11 @@ gcal_import_dialog_class_init (GcalImportDialogClass *klass)
   gtk_widget_class_bind_template_child (widget_class, GcalImportDialog, headerbar);
   gtk_widget_class_bind_template_child (widget_class, GcalImportDialog, import_button);
   gtk_widget_class_bind_template_child (widget_class, GcalImportDialog, placeholder_spinner);
-  gtk_widget_class_bind_template_child (widget_class, GcalImportDialog, preferences_page);
+  gtk_widget_class_bind_template_child (widget_class, GcalImportDialog, calendars_box);
   gtk_widget_class_bind_template_child (widget_class, GcalImportDialog, title_sizegroup);
+  gtk_widget_class_bind_template_child (widget_class, GcalImportDialog, toast_overlay);
 
   gtk_widget_class_bind_template_callback (widget_class, on_calendar_combo_row_selected_item_changed_cb);
-  gtk_widget_class_bind_template_callback (widget_class, on_cancel_button_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_import_button_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_combo_row_signal_factory_bind_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_combo_row_signal_factory_setup_cb);
@@ -519,10 +603,34 @@ gcal_import_dialog_new_for_files (GcalContext  *context,
 
   self =  g_object_new (GCAL_TYPE_IMPORT_DIALOG,
                         "context", context,
-                        "use-header-bar", TRUE,
                         NULL);
 
   setup_files (self, files, n_files);
 
   return GTK_WIDGET (self);
 }
+
+GtkWidget*
+gcal_import_dialog_new_for_file_list (GcalContext *context,
+                                      GList       *file_list)
+{
+  GcalImportDialog *self;
+
+  self =  g_object_new (GCAL_TYPE_IMPORT_DIALOG,
+                        "context", context,
+                        NULL);
+
+  setup_files_list (self, file_list);
+
+  return GTK_WIDGET (self);
+}
+
+void
+gcal_import_dialog_add_toast (GcalImportDialog *self,
+                              AdwToast         *toast)
+{
+  g_return_if_fail (GCAL_IS_IMPORT_DIALOG (self));
+  g_return_if_fail (ADW_IS_TOAST (toast));
+
+  adw_toast_overlay_add_toast (self->toast_overlay, toast);
+}
