@@ -38,6 +38,8 @@ struct _GcalImportFileRow
   GFile              *file;
   GPtrArray          *ical_components;
   GPtrArray          *ical_timezones;
+
+  GcalContext        *context;
 };
 
 static void          read_calendar_finished_cb                   (GObject            *source_object,
@@ -49,6 +51,7 @@ G_DEFINE_TYPE (GcalImportFileRow, gcal_import_file_row, ADW_TYPE_BIN)
 enum
 {
   PROP_0,
+  PROP_CONTEXT,
   PROP_FILE,
   N_PROPS,
 };
@@ -113,7 +116,9 @@ fill_grid_with_event_data (GcalImportFileRow *self,
   g_autofree gchar *description = NULL;
   g_autofree gchar *end_string = NULL;
   g_autoptr (GDateTime) start = NULL;
+  g_autoptr (GDateTime) start_local = NULL;
   g_autoptr (GDateTime) end = NULL;
+  g_autoptr (GDateTime) end_local = NULL;
   ICalTime *ical_start;
   ICalTime *ical_end;
   gint row = 0;
@@ -123,24 +128,45 @@ fill_grid_with_event_data (GcalImportFileRow *self,
   if (i_cal_time_is_date (ical_start))
     start_string = g_date_time_format (start, "%x");
   else
-    start_string = g_date_time_format (start, "%x %X");
+    {
+      start_local = g_date_time_to_local (start);
+
+      switch (gcal_context_get_time_format (self->context))
+        {
+        case GCAL_TIME_FORMAT_24H:
+          start_string = g_date_time_format (start_local, "%x %R");
+          break;
+
+        case GCAL_TIME_FORMAT_12H:
+          start_string = g_date_time_format (start_local, "%x %I:%M %P");
+          break;
+
+        default:
+          g_assert_not_reached ();
+        }
+    }
 
   ical_end = i_cal_component_get_dtend (ical_component);
-  if (ical_end)
-    {
-      end = gcal_date_time_from_icaltime (ical_end);
-      if (i_cal_time_is_date (ical_end))
-        end_string = g_date_time_format (end, "%x");
-      else
-        end_string = g_date_time_format (end, "%x %X");
-    }
+  end = gcal_date_time_from_icaltime (ical_end);
+  if (i_cal_time_is_date (ical_end))
+    end_string = g_date_time_format (end, "%x");
   else
     {
-      end = g_date_time_add_days (start, 1);
-      if (i_cal_time_is_date (ical_start))
-        end_string = g_date_time_format (end, "%x");
-      else
-        end_string = g_date_time_format (end, "%x %X");
+      end_local = g_date_time_to_local (end);
+
+      switch (gcal_context_get_time_format (self->context))
+        {
+        case GCAL_TIME_FORMAT_24H:
+          end_string = g_date_time_format (end_local, "%x %R");
+          break;
+
+        case GCAL_TIME_FORMAT_12H:
+          end_string = g_date_time_format (end_local, "%x %I:%M %P");
+          break;
+
+        default:
+          g_assert_not_reached ();
+        }
     }
 
   gcal_utils_extract_google_section (i_cal_component_get_description (ical_component),
@@ -281,7 +307,7 @@ read_calendar_finished_cb (GObject      *source_object,
   add_events_to_listbox (self, event_components);
 
   self->ical_components = g_ptr_array_ref (event_components);
-  self->ical_timezones = g_ptr_array_ref (timezones);
+  self->ical_timezones = g_steal_pointer (&timezones);
 
   g_signal_emit (self, signals[FILE_LOADED], 0, event_components);
 }
@@ -298,6 +324,7 @@ gcal_import_file_row_finalize (GObject *object)
 
   g_cancellable_cancel (self->cancellable);
   g_clear_object (&self->cancellable);
+  g_clear_object (&self->context);
   g_clear_object (&self->file);
   g_clear_pointer (&self->ical_components, g_ptr_array_unref);
   g_clear_pointer (&self->ical_timezones, g_ptr_array_unref);
@@ -315,6 +342,10 @@ gcal_import_file_row_get_property (GObject    *object,
 
   switch (prop_id)
     {
+    case PROP_CONTEXT:
+      g_value_set_object (value, self->context);
+      break;
+
     case PROP_FILE:
       g_value_set_object (value, self->file);
       break;
@@ -334,6 +365,11 @@ gcal_import_file_row_set_property (GObject      *object,
 
   switch (prop_id)
     {
+    case PROP_CONTEXT:
+      g_assert (self->context == NULL);
+      self->context = g_value_dup_object (value);
+      break;
+
     case PROP_FILE:
       g_assert (self->file == NULL);
       self->file = g_value_dup_object (value);
@@ -364,6 +400,12 @@ gcal_import_file_row_class_init (GcalImportFileRowClass *klass)
                                        1,
                                        G_TYPE_PTR_ARRAY);
 
+  properties[PROP_CONTEXT] = g_param_spec_object ("context",
+                                                  "Context",
+                                                  "Context",
+                                                  GCAL_TYPE_CONTEXT,
+                                                  G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
+
   properties[PROP_FILE] = g_param_spec_object ("file",
                                                "An ICS file",
                                                "An ICS file",
@@ -386,12 +428,14 @@ gcal_import_file_row_init (GcalImportFileRow *self)
 }
 
 GtkWidget*
-gcal_import_file_row_new (GFile        *file,
+gcal_import_file_row_new (GcalContext  *context,
+                          GFile        *file,
                           GtkSizeGroup *title_sizegroup)
 {
   GcalImportFileRow *self;
 
   self = g_object_new (GCAL_TYPE_IMPORT_FILE_ROW,
+                       "context", context,
                        "file", file,
                        NULL);
   self->title_sizegroup = title_sizegroup;
