@@ -112,12 +112,14 @@ allocate_overflow_popover (GcalMonthView *self,
 {
   graphene_point_t origin, end;
   GtkAllocation popover_allocation;
-  GtkAllocation allocation;
+  GtkAllocation cell_allocation;
   gint popover_min_width;
   gint popover_nat_width;
   gint popover_height;
   gint popover_width;
   gint header_height;
+  gint popover_x2;
+  gint popover_y2;
 
   g_assert (self->overflow.relative_to != NULL);
 
@@ -136,7 +138,7 @@ allocate_overflow_popover (GcalMonthView *self,
                                  &end))
     g_assert_not_reached ();
 
-  allocation = (GtkAllocation) {
+  cell_allocation = (GtkAllocation) {
     .x = origin.x,
     .y = origin.y,
     .width = end.x - origin.x,
@@ -157,21 +159,25 @@ allocate_overflow_popover (GcalMonthView *self,
                       &popover_height,
                       NULL, NULL);
 
-  popover_width = CLAMP (popover_nat_width, popover_min_width, allocation.width * 1.5);
-  popover_height = CLAMP (popover_height, allocation.height * 1.5, height);
+  popover_width = CLAMP (popover_nat_width,
+                         MAX (popover_min_width, cell_allocation.width * 1.25),
+                         cell_allocation.width * 1.5);
+  popover_height = CLAMP (popover_height, cell_allocation.height * 1.5, height);
 
   popover_allocation = (GtkAllocation) {
-    .x = MAX (0, allocation.x - (popover_width - allocation.width) / 2.0),
-    .y = MAX (header_height, allocation.y - (popover_height - allocation.height) / 2.0),
+    .x = MAX (0, cell_allocation.x - (popover_width - cell_allocation.width) / 2.0),
+    .y = MAX (header_height, cell_allocation.y - (popover_height - cell_allocation.height) / 2.0),
     .width = popover_width,
     .height = popover_height,
   };
 
-  if (popover_allocation.x + popover_allocation.width > width)
-    popover_allocation.x -= (popover_allocation.x + popover_allocation.width - width);
+  popover_x2 = popover_allocation.x + popover_allocation.width;
+  if (popover_x2 > width)
+    popover_allocation.x -= (popover_x2 - width);
 
-  if (popover_allocation.y + popover_allocation.height > height)
-    popover_allocation.y -= (popover_allocation.y + popover_allocation.height - height);
+  popover_y2 = popover_allocation.y + popover_allocation.height;
+  if (popover_y2 > height)
+    popover_allocation.y -= (popover_y2 - height);
 
   gtk_widget_size_allocate (self->overflow.popover, &popover_allocation, baseline);
 }
@@ -807,7 +813,7 @@ on_click_gesture_released_cb (GtkGestureClick *click_gesture,
   selection_start = g_date_time_ref (selection_start);
   selection_end = g_date_time_add_days (selection_end, 1);
 
-  selection_range = gcal_range_new (selection_start, selection_end, GCAL_RANGE_DEFAULT);
+  selection_range = gcal_range_new (selection_start, selection_end, GCAL_RANGE_DATE_ONLY);
   gcal_view_create_event (GCAL_VIEW (self), selection_range, x, y);
 
   GCAL_EXIT;
