@@ -21,13 +21,12 @@
 #pragma once
 
 #include "gcal-event.h"
-
-#include <dazzle.h>
+#include "gcal-search-hit.h"
 
 G_BEGIN_DECLS
 
 #define GCAL_TYPE_SEARCH_HIT_EVENT (gcal_search_hit_event_get_type())
-G_DECLARE_FINAL_TYPE (GcalSearchHitEvent, gcal_search_hit_event, GCAL, SEARCH_HIT_EVENT, DzlSuggestion)
+G_DECLARE_FINAL_TYPE (GcalSearchHitEvent, gcal_search_hit_event, GCAL, SEARCH_HIT_EVENT, GcalSearchHit)
 
 GcalSearchHitEvent*  gcal_search_hit_event_new                   (GcalEvent          *event);
 
