@@ -32,6 +32,7 @@
 
 #include <libecal/libecal.h>
 #include <libedataserver/libedataserver.h>
+#include <libsoup/soup.h>
 
 #include <glib/gi18n.h>
 
@@ -1156,11 +1157,15 @@ typedef struct
 } AskRecurrenceData;
 
 static void
-on_message_dialog_response_cb (GtkDialog         *dialog,
-                               const gchar       *response,
-                               AskRecurrenceData *data)
+on_message_dialog_response_cb (GObject      *source_object,
+                               GAsyncResult *result,
+                               gpointer      user_data)
 {
   GcalRecurrenceModType mod_type;
+  AskRecurrenceData *data;
+  const char *response;
+
+  response = adw_alert_dialog_choose_finish (ADW_ALERT_DIALOG (source_object), result);
 
   if (g_strcmp0 (response, "this-only") == 0)
     mod_type = GCAL_RECURRENCE_MOD_THIS_ONLY;
@@ -1171,6 +1176,7 @@ on_message_dialog_response_cb (GtkDialog         *dialog,
   else
     mod_type = GCAL_RECURRENCE_MOD_NONE;
 
+  data = (AskRecurrenceData *) user_data;
   data->callback (data->event, mod_type, data->user_data);
   g_clear_object (&data->event);
   g_clear_pointer (&data, g_free);
@@ -1185,35 +1191,37 @@ gcal_utils_ask_recurrence_modification_type (GtkWidget                 *parent,
 {
   AskRecurrenceData *data;
   ECalClient *client;
-  GtkWidget *dialog;
+  AdwDialog *dialog;
 
   data = g_new0 (AskRecurrenceData, 1);
   data->event = g_object_ref (event);
   data->callback = callback;
   data->user_data = user_data;
 
-  dialog = adw_message_dialog_new (GTK_WINDOW (gtk_widget_get_native (parent)),
-                                   _("Modify Multiple Events?"),
-                                   _("The event you are trying to modify is recurring. The changes you have selected should be applied to:"));
+  dialog = adw_alert_dialog_new (_("Modify Multiple Events?"),
+                                 _("The event you are trying to modify is recurring. The changes you have selected should be applied to:"));
 
-  adw_message_dialog_add_responses (ADW_MESSAGE_DIALOG (dialog),
-                                    "close", _("_Cancel"),
-                                    "this-only", _("_Only This Event"),
-                                    NULL);
+  adw_alert_dialog_add_responses (ADW_ALERT_DIALOG (dialog),
+                                  "close", _("_Cancel"),
+                                  "this-only", _("_Only This Event"),
+                                  NULL);
 
   client = gcal_calendar_get_client (gcal_event_get_calendar (event));
 
   if (!e_client_check_capability (E_CLIENT (client), E_CAL_STATIC_CAPABILITY_NO_THISANDFUTURE))
-    adw_message_dialog_add_response (ADW_MESSAGE_DIALOG (dialog), "subsequent-events", _("_Subsequent Events"));
+    adw_alert_dialog_add_response (ADW_ALERT_DIALOG (dialog), "subsequent-events", _("_Subsequent Events"));
 
   if (show_mod_all)
-    adw_message_dialog_add_response (ADW_MESSAGE_DIALOG (dialog), "all-events",  _("_All Events"));
+    adw_alert_dialog_add_response (ADW_ALERT_DIALOG (dialog), "all-events",  _("_All Events"));
 
-  gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (gtk_widget_get_native (parent)));
+  adw_alert_dialog_set_default_response (ADW_ALERT_DIALOG (dialog), "cancel");
+  adw_alert_dialog_set_close_response (ADW_ALERT_DIALOG (dialog), "cancel");
 
-  g_signal_connect (dialog, "response", G_CALLBACK (on_message_dialog_response_cb), data);
-
-  gtk_window_present (GTK_WINDOW (dialog));
+  adw_alert_dialog_choose (ADW_ALERT_DIALOG (dialog),
+                           parent,
+                           NULL,
+                           on_message_dialog_response_cb,
+                           data);
 }
 
 /**
@@ -1289,3 +1297,90 @@ gcal_is_valid_event_name (const gchar *event_name)
   g_autofree gchar *aux = g_strstrip (g_strdup (event_name));
   return g_utf8_strlen (aux, -1) > 0;
 }
+
+/**
+ * gcal_get_service_name_from_url:
+ * @url: the meeting url to get service name from
+ *
+ * Given a meeting service URL, get the service provider name,
+ * or return NULL in the case the service is not listed.
+ *
+ * Returns: the service name or NULL
+ */
+const gchar *
+gcal_get_service_name_from_url (const gchar *url)
+{
+  struct {
+    const gchar *needle;
+    const gchar *service_name;
+  } service_name_vtable[] = {
+    { "meet.google.com", N_("Google Meet") },
+    { "meet.jit.si", N_("Jitsi") },
+    { "whereby.com", N_("Whereby") },
+    { "zoom.us", N_("Zoom") },
+    { "teams.microsoft.com", N_("Microsoft Teams") },
+  };
+  gsize i;
+
+  for (i = 0; i < G_N_ELEMENTS (service_name_vtable); i++)
+    {
+      if (strstr (url, service_name_vtable[i].needle))
+        return gettext (service_name_vtable[i].service_name);
+    }
+
+  return NULL;
+}
+
+/**
+ * gcal_create_soup_session:
+ *
+* Creates a new #SoupSession with correct default settings.
+ *
+ * Returns: (transfer full): a new #SoupSession
+ */
+SoupSession *
+gcal_create_soup_session (void)
+{
+  g_autoptr (SoupSession) session = NULL;
+
+  session = soup_session_new ();
+
+  /* Set up a 10 seconds default timeout, as 60 is too long for fail-fast performance in general.
+   * Individual parts of the code can always override this if needed. */
+  soup_session_set_timeout (session, 10);
+
+  /* RFC2616 states HTTP requests "should" contain the User-Agent header. In practice though, in our situation
+   * User-Agent is mandatory, or restrictive "web application firewalls" can do unexpected things with our request.
+   * The trailing " - " lets libsoup append its name and version for us. */
+  soup_session_set_user_agent (session, "GNOME Calendar/" PACKAGE_VERSION " - ");
+
+  return g_steal_pointer (&session);
+}
+
+/**
+ * gcal_create_writable_calendars_model:
+ * @manager: a #GcalManager
+ *
+ * Retrieves a model with all available read-write #GcalCalendar.
+ * This is useful for binding to combo rows.
+ *
+ * Returns: (transfer full): a #GListModel with all available read-write #GcalCalendar
+ */
+GListModel*
+gcal_create_writable_calendars_model (GcalManager *manager)
+{
+  GtkBoolFilter *bool_filter;
+  GtkExpression *expression;
+  GListModel *calendars;
+
+  g_return_val_if_fail (GCAL_IS_MANAGER (manager), NULL);
+
+  expression = gtk_property_expression_new (GCAL_TYPE_CALENDAR, NULL, "read-only");
+
+  bool_filter = gtk_bool_filter_new (expression);
+  gtk_bool_filter_set_invert (bool_filter, TRUE);
+
+  calendars = gcal_manager_get_calendars_model (manager);
+  return G_LIST_MODEL (gtk_filter_list_model_new (g_object_ref (calendars), GTK_FILTER (bool_filter)));
+}
+
